
!----------------------------------------------------------------------
!
!  Kyodo KG109 aka H40 aka Autonet-10 UHF HT
!
!  0.0 970507 First cut.
!  0.1 970510 Second.
!  0.2 970510 Third.
!
!  Juha, OH5NXO
!
#define VERSION "@(#)KG109 0.2 970510 PROM"


!----------------------------------------------------------------------
!
!  General notes
!
!  Some connectors have reversed numbering on different schematics.
!  Monitor-button also lights up the smaller LCD...
!  Backlight of the larger LCD is sourced from a 4099...
!

!----------------------------------------------------------------------
!
!  Generate HZ timer interrupt.
!  Reload OCR1 as quickly as reasonable, in the interrupt, to minimize skew ?
!
!  4.032 MHz Xtal
!  1.008 MHz E Clock
!
HZ      = 100
PROCCLK = 1008000
SYSTICK = (PROCCLK / HZ)

!----------------------------------------------------------------------
!
!  Ftx = Fvco
!  Frx = Fvco + 21.6 MHz
!
!  1st IF 21.6 MHz
!
!  Synth TC9181 (?)
!  Data in LSb first.
!  Prescaler 64/65.
!  Reference 6 MHz.
!  Two last (highest, MSb) bits control where the data goes ?
!
!  0b10 1 RRRRRRRRRRRRR       set "channel spacing", last 1 bit ???
!              13
!
!  0b0000                     unknown meaning ? external SWx pins ?
!     4
!
!  0b01 NNNNNNNNNNN 0 AAAAAA  set "frequency"
!         11            6
!
!  NNNNNNNNNNAAAAAA is straight Fvco / 12.5, nice.
!  Use R of 480 for 12.5
!
!  Shift from LSb, first 6 bits (A), then one bit of zeropad,
!  then rest of divisor (11 bits, N) and finally 0b01. Strobe it.
!


!----------------------------------------------------------------------
!
!  Control register, IC11 4099
!  Notice how address bus A5,A6,A7 is reversed to A2,A1,A0 in decoder.
!  IC11 data is DB7.
!

ONEBIT     = 0x80
ZEROBIT    = 0x00

MDM_TXEN   = 0x4000 ! 0      FX419
CLOCK      = 0x4020	! 1      PCF2100, synth, e2prom clock
MUTE       = 0x4040 ! 2 CN3P
PWR_SAVE   = 0x4060 ! 3 CN3P
LCD_LIGHT  = 0x4080 ! 4 CN5  aka SPARE O/P
DATA       = 0x40A0	! 5      PCF2100, synth, e2prom write
DLEN       = 0x40C0 ! 6      PCF2100
SYN_STROBE = 0x40E0 ! 7 CN3P

!
!  Pin assignments
!
!  P6 generic pins 0, 4 and 5 inputs, P6DDR 0xCE
!
!  P60         in  RESET input (just logical...)
!  P61         out e2prom1enb
!  P62         out e2prom2enb
!  P63         out e2prom3enb
!  P64         in  voltage comparator 1
!  P65         in  voltage comparator 2
!  P66         out modulation off
!  P67         out PTT_OUT
!
!  P2 generic pins all inputs, P2DDR 0x00, x for implied output...
!                                          SCL i/o really
!
!  P20 (Tin)   in  RXSYNC from modem, generate Input Capture Interrupt
!  P21 (Tout1) i/o SCL to PCD3312 and KEYPAD, to Molex-2
!  P22 (SCLK)  in  SCAN
!  P23 (Rx)    in  RS-232 IN
!  P24 (Tx)    inx RS-232 OUT
!  P25 (Tout2) in  UP
!  P26 (Tout3) inx SIDETONE, Toggles From Timer 2, occasionally
!  P27 (TCLK)  in  DOWN
!
!  P5 generic pin 4 output, P5DDR 0x10 or 0x11 when iicing.
!
!  P50 (/IRQ1) i/o SDA to PCD3312 and KEYPAD, e2prom read
!  P51 (/IRQ2) in  TXSYNC from modem, generates Interrupt 2
!  P52 (MR)    in  DCD from modem
!  P53 (/HALT) in  RXDATA from modem
!  P54 (/IS)   out TXDATA to modem
!  P55 (/OS)   in  BUSY
!  P56         in  PTT-1 and PTT-2
!  P57         in  PTT-1 and CALL
!

!----------------------------------------------------------------------
!
!  Make the IIC bit-fiddling somewhat more readable
!  Portpin is either output and pulled low, or input. There is pullup.
!  So setting the DDR is all we need to do iic.
!

#define SDA_BIT    bit @PORT5, $ SDA ; jsr iic_delay
#define SDA_PULL   and @PORT5, $~SDA ; jsr iic_delay
#define SDA_HIGH   push a; ld a, $0x10; st a, @P5DDR; pop a; jsr iic_delay
#define SDA_LOW    push a; ld a, $0x11; st a, @P5DDR; pop a; jsr iic_delay

#define SCL_BIT    bit @PORT2, $ SCL ; jsr iic_delay
#define SCL_PULL   and @PORT2, $~SCL ; jsr iic_delay
#define SCL_HIGH   push a; ld a, $0x00; st a, @P2DDR; pop a; jsr iic_delay
#define SCL_LOW    push a; ld a, $ SCL; st a, @P2DDR; pop a; jsr iic_delay

!----------------------------------------------------------------------
!
!  Beginning of RAM is masked by internal registers and storage.
!  There can be either 2 or 8 kB RAM.
!

#define BYTE   : .rs 1
#define WORD   : .rs 2
#define BUF(n) : .rs (n)

	.data
	.org 0x0000

	ASSERT(. == 0x00)

P1DDR	BYTE	! 00 WO                                 FE
P2DDR	BYTE	! 01 WO                                 00
PORT1	BYTE	! 02 RW                                 ??
PORT2	BYTE	! 03 RW                                 ??

	SCL = 0x02	! PCF3312, PCF8574 and PCF8576

P3DDR	BYTE	! 04 WO                                 FE
P4DDR	BYTE	! 05 WO                                 00
PORT3	BYTE	! 06 RW                                 ??
PORT4	BYTE	! 07 RW                                 ??

TCSR1	BYTE	! 08 RW Timer Control/Status Register 1 00

	ICF   = 0x80
	OCF1  = 0x40
	TOF   = 0x20
	EICI  = 0x10
	EOCI1 = 0x08
	ETOI  = 0x04
	IEDG  = 0x02
	OLVL1 = 0x01

FRC		WORD	! 09 RW Free Running Counter            0000
OCR1	WORD	! 0B RW Output Compare Register 1       FFFF
ICR		WORD	! 0D RO Input Capture Register          0000
TCSR2	BYTE	! 0F RW Timer Control/Status Register 2 10

	OCF2   = 0x20
	EOICI2 = 0x08
	OLVL2  = 0x04
	OE2    = 0x02
	OE1    = 0x01

RMCR	BYTE	! 10 RW Rate/Mode Control Register      C0

TRCSR1	BYTE	! 11 RW Tx/Rx Control/Status Register 1 20
RDR		BYTE	! 12 RO Receive Data Register           00
TDR		BYTE	! 13 WO Transmit Data Register          ??

RP5CR	BYTE	! 14 RW RAM/PORT5 Control Register      F8 or 78

	STBY_PWR  = 0x80
	RAME      = 0x40
	STBY_FLAG = 0x20
	IRQ2E     = 0x02
	IRQ1E     = 0x01

PORT5	BYTE	! 15 RW                                 ??

	SDA = 0x01	! PCF3312, PCF8574 and PCF8576

P6DDR	BYTE	! 16 WO                                 00
PORT6	BYTE	! 17 RW                                 ??

	MONI = 0x01	! aka DEF MUTE or RESET

PORT7	BYTE	! 18 RW                                 ??

OCR2	WORD	! 19 RW Output Compare Register 2       FFFF
TCSR3	BYTE	! 1B RW Timer Control/Status Register 3 20

	CMF  = 0x80 ! Counter Match Flag, read-only
	ECMI = 0x40 ! Enable Counter Match Interrupt
	T2E  = 0x10 ! Timer 2 Enable
	TOS1 = 0x08 ! Timer Output Select
	TOS0 = 0x04
	CKS1 = 0x02 ! Input Clock Select
	CKS0 = 0x01

TCONR	BYTE	! 1C WO Time Constant Register          FF
T2CNT	BYTE	! 1D RW Timer 2 Up Counter              00
TRCSR2	BYTE	! 1E RW Tx/Rx Control/Status Register 2 28

TSTREG	BYTE	! 1F -- KEEP OFF !!!

P5DDR	BYTE	! 20 WO                                 00
P6CSR	BYTE	! 21 RW PORT 6 Control/Status Register  07

	ASSERT(. == 0x22)

!
!  Hole to external RAM
!
	.org 0x0028

!
!  Internal RAM
!
	.org 0x0040

tps	BYTE			! Hz counter

!
!  Rest or external RAM
!
	.org 0x0140

_bss:

indicators	BYTE	! BUSY BATT TX SCAN in upper LCD

	BATT = 0x1
	BUSY = 0x2
	TX   = 0x4
	SCAN = 0x8

dpychannel	WORD	! two digits in small LCD

digno	BYTE

dpydigits	BUF(20)	! 160 bits of big LCD

!
!  Stack from end of RAM (just 2kB for now, there might be 8kB SRAM)
!
	.org 0x07FF
stack:

_end:

!----------------------------------------------------------------------
!
!  EPROM is in last 16 kB
!
	.text
	.org 0xC000

version: .asciz VERSION

!
!  BUSY=b1        b1       b5
!               b9  bA   bD  bE     b3 and b7 unused
!                 b2       b6
!  BATT=b0      b8  bB   bC  bF
!                 b0       b4
!
!   TX=b2   SCAN=b3
!
!
!	                FEDCBA9876543210
	SMALL_blank = 0b0000000000000000
	SMALL_Er    = 0b0001001101000111

hextbl_10:
	.word 0b0000111100000011	! 0  Left digit of upper LCD
	.word 0b0000110000000000	! 1
	.word 0b0000010100000111	! 2
	.word 0b0000110000000111	! 3
	.word 0b0000111000000100	! 4
	.word 0b0000101000000111	! 5
	.word 0b0000101100000111	! 6
	.word 0b0000110000000010	! 7
	.word 0b0000111100000111	! 8
	.word 0b0000111000000111	! 9
	.word 0b0000111100000110	! A
	.word 0b0000101100000101	! b
	.word 0b0000001100000011	! C
	.word 0b0000110100000101	! d
	.word 0b0000001100000111	! E
	.word 0b0000001100000110	! F

hextbl_1:
	.word 0b1111000000110000	! 0  Right digit of upper LCD
	.word 0b1100000000000000	! 1
	.word 0b0101000001110000	! 2
	.word 0b1100000001110000	! 3
	.word 0b1110000001000000	! 4
	.word 0b1010000001110000	! 5
	.word 0b1011000001110000	! 6
	.word 0b1100000000100000	! 7
	.word 0b1111000001110000	! 8
	.word 0b1110000001110000	! 9
	.word 0b1111000001100000	! A
	.word 0b1011000001010000	! b
	.word 0b0011000000110000	! C
	.word 0b1101000001010000	! d
	.word 0b0011000001110000	! E
	.word 0b0011000001100000	! F

	nop
	nop
	nop		! make sure following code aligns ok, for naive dis6303.

!----------------------------------------------------------------------
!
!  Initialize synthesizer for 12.5 kHz spacing
!
init_synth:

	ld d, $ 480					! 6MHz / 12.5 kHz
	ld x, $13
	jsr sendit

	ld b, $ 0b101
	ld x, $3
	jsr sendit
	jsr strobe_synth			! set spacing

	ld b, $ 0b0000
	ld x, $4
	jsr sendit
	jsr strobe_synth			! unknown sequence

	rts

!----------------------------------------------------------------------
!
!  Generate load pulse for synthesizer
!
strobe_synth:
	ld a, $ONEBIT
	st a, *SYN_STROBE
	ld a, $ZEROBIT
	st a, *SYN_STROBE
	rts

!----------------------------------------------------------------------
!
!  d has channel as NP+A
!  64/65 allows us to skip the division.
!
rx_synth:
	sub d, $ 21600 / 25 * 2		! Account for IF

tx_synth:

send_vco:
	ld x, $6
	jsr sendit		! A5..A0

	jsr zerobit		! A6

	ld x, $10
	jsr sendit		! N9..N0

	jsr zerobit		! N10

	ld b, $ 0b01
	ld x, $2
	jsr sendit		! Direct them to divider

	jsr strobe_synth
	rts

!----------------------------------------------------------------------
!
!  Update big (lower) LCD
!
update_big_lcd:

	jsr iic_start

	!  Slave address
	!  0 1 1 1 0 0 SA0 R/W
	ld a, $ 0b01110000
	jsr iic_byte

	!  MODE SET
	!  C 1 0 LP E B M1..M0
	!  1 1 0 0  1 1 0 0
	ld a, $ 0b11001100
	jsr iic_byte

	!  LOAD DATA POINTER
	!  C 0 P5..P0
	!  1 0 0 0 0 0 0 0
	ld a, $ 0b10000000
	jsr iic_byte

	!  DEVICE SELECT
	!  C 1 1 0 0 A2..A0
	!  1 1 1 0 0 0 0 0
	ld a, $ 0b11100000
	jsr iic_byte

	!  BANK SELECT
	!  C 1 1 1 1 0 I O
	!  1 1 1 1 1 0 0 0
	ld a, $ 0b11111000
	jsr iic_byte

	!  BLINK
	!  C 1 1 1 0 A BF1..BF0
	!  0 1 1 1 0 0 0 0
	ld a, $ 0b01111000
	jsr iic_byte

	ld x, $dpydigits
1:
	ld a, *x
	inc x
	jsr iic_byte
	cmp x, $(dpydigits + SIZE(dpydigits))
	bnz 1b

	jsr iic_stop

	rts

!----------------------------------------------------------------------
!
!  Remember the LCD might be in powersaving state...
!  This is too much in any case.
!
iic_delay:
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

	rts

iic_start:

	SDA_HIGH
	SCL_HIGH				! Set them free

	SDA_PULL				! output state low, when DDR set
	SCL_PULL
1:
	SDA_BIT
	bz 1b
1:
	SCL_BIT
	bz 1b					! Wait iic idle

	SDA_LOW					! START condition
	SCL_LOW					! both kept low

	rts

iic_stop:

	SDA_LOW					!
	SCL_HIGH				! Let them go for
1:
	SCL_BIT
	bz 1b

	SDA_HIGH				! STOP condition

	rts

iic_byte:
	
	bsr iic_bit
	bsr iic_bit
	bsr iic_bit
	bsr iic_bit

	bsr iic_bit
	bsr iic_bit
	bsr iic_bit
	bsr iic_bit

	bsr iic_ackbit			! acknowledge from receiver
	rts

iic_bit:
	rlc a
	bnc iic_zerobit
	br  iic_onebit

iic_zerobit:

	SDA_LOW		! lower SDA
	br iic_dobit

iic_onebit:

	SDA_HIGH
	br iic_dobit

iic_ackbit:
	
	SDA_HIGH
1:
	SDA_BIT
	bnz 1b		! wait ack
	br iic_dobit

iic_dobit:

	SCL_HIGH
1:
	SCL_BIT
	bz 1b		! wait to go up

	SCL_LOW

	rts

!----------------------------------------------------------------------
!
!  Update small (upper) LCD
!
update_small_lcd:

	ld a, $ZEROBIT
	st a, *DLEN
	st a, *DATA

	ld a, $ONEBIT
	st a, *DLEN

	jsr zerobit		! leading zerobit

	ld b, *indicators
	ld x, $4
	jsr sendit		! BUSY BATT TX SCAN

	ld b, *(dpychannel + 0)
	ld x, $8
	jsr sendit		! First group of twodigit channel

	ld d, $0xFF
	ld x, $8
	jsr sendit		! Padding

	jsr onebit		! This was group 1

	ld a, $ZEROBIT
	st a, *DLEN
	st a, *DATA

	jsr clockit		! load pulse for group 1

	ld a, $ONEBIT
	st a, *DLEN

	jsr zerobit		! leading zerobit

	ld b, $0xF
	ld x, $4
	jsr sendit		! Padding

	ld b, *(dpychannel + 1)
	ld x, $8
	jsr sendit		! Second group of twodigit channel

	ld d, $0xFF
	ld x, $8
	jsr sendit		! Padding

	jsr zerobit		! This was group 0

	ld a, $ZEROBIT
	st a, *DLEN
	st a, *DATA

	jsr clockit		! load pulse for group 0

	rts

clockit:
	push a
	ld a, $ONEBIT
	st a, *CLOCK
	ld a, $ZEROBIT
	st a, *CLOCK
	pop a
	rts

!----------------------------------------------------------------------
!
!  x lowest bits from d into synth or small lcd, LSb first, max 16 bits.
!
sendit:
1:
	lsr d			! LSb into carry
	push a			! save d, carry kept
	rrc a			! carry into bit7
	st a, *DATA		! bit to dataline
	ld a, $ONEBIT
	st a, *CLOCK	! raise and ...
	ld a, $ZEROBIT
	st a, *CLOCK	! lower clock
	pop a
	dec x
	bnz 1b
	rts

zerobit:
	push a			! keep all
	ld a, $ZEROBIT
	br 1f

onebit:
	push a			! keep all
	ld a, $ONEBIT
1:
	st a, *DATA		! bit to dataline
	ld a, $ONEBIT
	st a, *CLOCK	! raise and ...
	ld a, $ZEROBIT
	st a, *CLOCK	! lower clock
	pop a
	rts

!----------------------------------------------------------------------
!
!  RXSYNC from modem generated Input Capture for Timer 1
!  Dummy routine for now.
!
_ici:
	ld a, @TCSR1	! read and ...
	ld d, @ICR		! discard these, rearm ICI.

	rti

!----------------------------------------------------------------------
!
!  TXSYNC from modem comes to /IRQ2
!  Dummy routine for now.
!
_irq2:
	rti

!----------------------------------------------------------------------
!
!  timer tick from Output Compare 1
!
_oci:
	ld a, @TCSR1	! discard this for rearming OCI1.
	ld d, @OCR1
	add d, $SYSTICK
	st d, @OCR1		! rearm and set match after next SYSTICKs. Slight skew ?

	!
	!  And now systick() things.
	!

	ld a, @tps
	inc a
	st a, @tps
	cmp a, $ HZ
	bnz 1f
	clr a
	st a, @tps

	jsr update_small_lcd
1:
	rti

!----------------------------------------------------------------------
!
!  Timer 2 Counter Match Interrupt.
!  Could be used to flap other pins (SIDETONE in P26 is automatic).
!
_cmi:
	and @TCSR3, $~CMF	! clear CMF, rearm CMI.
	rti

!----------------------------------------------------------------------
!
!  Kinda Dr Allcome.
!
_irq1:
_sio:
_toi:
_swi:
_nmi:
_trap:
	jmp _start

!----------------------------------------------------------------------
!
!   Reset vector
!
_start:

	sei
	ld sp, $stack

	ld a, $0xCE
	st a, @P6DDR
	ld a, $0x00
	st a, @PORT6		! Tx OFF immediately

	ld a, $0x10
	st a, @P5DDR
	st a, @PORT5

	ld a, $0x00
	st a, @P2DDR

	!
	!  Clear the output-latch
	!
	ld a, $ZEROBIT

	st a, *SYN_STROBE
	st a, *PWR_SAVE
	st a, *MDM_TXEN
	st a, *LCD_LIGHT
	st a, *CLOCK
	st a, *DATA
	st a, *DLEN
	st a, *MUTE

	!
	!  /HALT disabled
	!  /IRQ1 disabled
	!  "Memory ready" disabled
	!  Internal RAM and /IRQ2 enabled
	!
	ld a, $(STBY_PWR | RAME | STBY_FLAG | IRQ2E)
	st a, @RP5CR

	!
	!  Timer tick and modem rxsync come from Timer 1
	!  Tick from Output Compare 1 and RXSYNC from Input Capture.
	!  Use IEDG = "1" for rising edge RXSYNC XXX
	!
	ld a, @TCSR1			! Try resetting interrupt logic
	ld a, @TCSR2			! by reading the statuses
	ld a, @TCSR3			! of all timer registers
	and @TCSR3, $~CMF		! and clear CMF.

	ld a, $SYSTICK
	st a, @OCR1

	ld a, $(EOCI1 | EICI)
	st a, @TCSR1

	!
	!  Timer 2 is used for sidetone.
	!  Interrupt is disabled.
	!  Input is (FRC of Timer 1) / 128.
	!  Output Select is to toggle SIDETONE.
	!  Counter is stopped when not generating keyclicks or alarms.
	!  Use or @TCSR3, $T2E to start, and and @TCSR3, $~T2E to stop tone.
	!  Use ECMI to get _cmi also, in addition to the toggling.
	!
	!  4032000 Hz CLK
	!  1008000 Hz E CLK
	!
	!   126000 Hz Timer 2 input for /= 8 (CKSx)
	!	  1750 Hz from constant 72
	!	  3500 Hz from constant 36  (this for beep)
	!      494 Hz from constant 255

	ld d, $(PROCCLK / 8 / (2 * 1750))
	st d, @TCONR

	ld a, $(CKS1 | TOS0)
	st a, @TCSR3

	!
	!  Could configure the Serial Communication Interface here...
	!

	!
	!  Wait and clear zero data.
	!

	ld x, $ 0
1:
	dec x
	bnz 1b

	ld x, $_bss
	clr a
1:
	st a, *x
	inc x
	cmp x, $_end
	bnz 1b

	!
	!  End of initialization.
	!

	jsr init_synth

	ld d, $ 433500 / 25 * 2
	jsr rx_synth

	ld b, $0xF
	st b, *indicators
	ld d, $ SMALL_Er
	st d, *dpychannel

	jsr update_small_lcd
	jsr update_big_lcd

	cli
	jmp main

!----------------------------------------------------------------------
!
!  Main commutator loop
!
main:

	bit @PORT6, $MONI
	bz 1f
	ld a, $ONEBIT
	st a, *LCD_LIGHT
	jsr hack
	ld a, *digno
	jsr drawhex
	jsr update_big_lcd
	jsr update_small_lcd
2:
	bit @PORT6, $MONI
	bnz 2b
	ld a, $ZEROBIT
	st a, *LCD_LIGHT
1:

	cli
	slp				! energy star compliance
	slp				! energy star compliance
	jmp main

drawhex:

	push a
	ld b, a
	lsr b
	lsr b
	lsr b
	lsr b
	ld x, $hextbl_10
	asl b
	add x, b
	ld d, *x
	st d, *dpychannel

	pop b
	and b, $0xF
	ld x, $hextbl_1
	asl b
	add x, b
	ld d, *dpychannel
	or a, x[0]
	or b, x[1]
	st d, *dpychannel

	rts

hack:
	inc *digno
	ld x, $dpydigits
1:
	ld a, *x
	bnz 1f
	inc x
	cmp x, $(dpydigits + SIZE(dpydigits))
	bz 2f
	br 1b
1:
	lsr a
	bnc 1f
	ld a, $0x80
	inc x
	cmp x, $(dpydigits + SIZE(dpydigits))
	bz 2f
1:
	st a, *x
	rts
2:
	ld a, $0x80
	st a, *dpydigits
	ld a, $0
	st a, *digno
	rts

	!
	!  Hello Fylde ppl !
	!
	.ascii "\nIndeed, rather nosy, I would say, yes.\n"

	.org 0xFFE8

.word 0x0000 ! FFE8
.word _irq2  ! FFEA /IRQ2                                 txsync from modem
.word _cmi   ! FFEC CMI   Timer 2 Counter Match           maybe for 1750Hz
.word _trap  ! FFEE TRAP
.word _sio   ! FFF0 SIO   RDRF + ORFE + TDRE + PER
.word _toi   ! FFF2 TOI   Timer 1 Overflow
.word _oci   ! FFF4 OCI   Timer 1 Output Compare 1, 2     system timer tick
.word _ici   ! FFF6 ICI   Timer 1 Input Capture           rxsync from modem
.word _irq1  ! FFF8 /IRQ1
.word _swi   ! FFFA SWI   Software Interrupt
.word _nmi   ! FFFC /NMI
.word _start ! FFFE /RES  RESET

	ASSERT(. == 0)	! overflow of 0x10000

